#
# bibliography package for Perl
#
# CSA - California Scientific Abstracts
#
# It seems that CSA uses an Ovid like format.  Like ovid, the most 
# important part is the "Source", which is different for every database.
# My solution to this is to match on the database name and parse the source
# accordingly.  This, however, leads to a never ending search for more 
# database types.
#
# This conversion module is somewhat fragile due to the weak parsing of the
# source section of CSA references.  
#
# When in doubt, consult the source - the reference Source that is.
#
# Copyright David M. Kaplan 2002
# License: GPL
#

package bp_CSA;

$version = "CSA (dmk 10 aug 2003)";

######

&bib::reg_format(
  'CSA',     # name
  'CSA',        # short name
  'bp_CSA',  # package name
  'none',       # default character set
  'suffix is CSA',
# our functions
  'options is standard',
  'open is standard',
  'close is standard',
  'read',
  'write is standard',
  'clear is standard',
  'explode',
  'implode is unsupported',
  'tocanon',
  'fromcanon is unsupported',
);

######################################################################
# Here I define the correspondence between CSA fields and canonical fields.
######################################################################

%csa_special = ( "SO" => "CSA_Source", 
		 "DN" => "Database_Name", 
		 "PT" => "Publication_Type" );
%csa_to_canon = ( 
		  "AU" => "Authors",
		  "AF" => "AuthorAddress",
		  "IS" => "ISSN",
		  "NT" => "Note",
		  "AB" => "Abstract",
		  "PY" => "Year",
		  "DE" => "Keywords",
		  "TI" => "Title"
		  );

%csa_types = ( "Journal Article" => "article",
	       "Article" => "article",
	       "Technical rept" => "report"
	       );

%csa_databases_s = ( "Oceanic Abstracts" => 
		     "parse_oceanic_source",
		     "Meteorological & Geoastrophysical Abstracts" =>
		     "parse_oceanic_source", 
		     "ASFA: Aquatic Sciences and Fisheries Abstracts" =>
		     "parse_oceanic_source" 
		   );

%csa_databases_n = ( "Oceanic Abstracts" => 
		     "parse_oceanic_names",
		     "Meteorological & Geoastrophysical Abstracts" =>
		     "parse_oceanic_names", 
		     "ASFA: Aquatic Sciences and Fisheries Abstracts" =>
		     "parse_oceanic_names" 
		   );

%csa_months = ( "jan" => "january",
		"feb" => "February",
		"mar" => "March",
		"apr" => "April",
		"may" => "May",
		"jun" => "June",
		"jul" => "July",
		"aug" => "August",
		"sep" => "September",
		"oct" => "October",
		"nov" => "November",
		"dec" => "December"
		);

######################################################################
# This function breaks up records on "Record # ..."
######################################################################

sub read {
  local($file) = @_;
  local($ent) = '';

  while (<$bib::glb_current_fh>) {
      next if /^\s*$/; # Skip over empty lines.

      # Deal with start of a new record.
      if (/^Record\s+([0-9]+)/) {
	  if ($1 == 1) {
	      $ent = ''; # This gets rid of preamble.
	  } else {
	      last; # This ends on new record.
	  }
      }

      $ent .= $_; # Add on line.
  }

  $ent;
}


######################################################################
# Breaks down entries into non-canonical fields.
######################################################################

sub explode {
  my (%entry) = ();
  my ($field) = undef;
  my ($ent) = shift;

  foreach (split(/\n/,$ent)) {
      if (/^\s+\S/) {
	  return &bib::goterror("CSA explode--Problems parsing entry") unless defined $field;

	  s/^\s+//; # Strip spaces at beginning of line.

	  # Add line to entry.
	  $entry{$field} && ( $entry{$field} .= " " . $_ ) || 
	      ( $entry{$field} = $_ );

	  next;
      } else { # Deal with new field type.
	  /^([A-Z][A-Z0-9]):\s+(.+)\s*$/;

	  if (defined( $csa_to_canon{$1} ) || defined( $csa_special{$1} )) {
	      $field = $1; # If it is known, use the short name.
	  } else {
	      $field = $2; # Otherwise, use the human readable name.
	      $field =~ s/\s/_/g; # Get rid of nasty spaces.
	  }

      }
  }
  
  %entry;
}

######


sub implode {
  local(%entry) = @_;
  return &bib::goterror("CSA implode isn't supported.");
}

######################################################################
# Turns fields produced by explode into canonical fields.
######################################################################
sub tocanon {
  my(%entry) = @_;
  my(%can,%s,$k);

  $entry{"PT"} =~ s/^\s+//; # Strip unneeded spaces.
  $entry{"PT"} =~ s/\s+$//;
  
  # Deal with special cases.
  $can{"CiteType"} = parse_csa_type( $entry{"PT"} );
  %s = parse_csa_source( $entry{"SO"}, $entry{"DN"}, $entry{"PT"} );

  # Append new information
  foreach (keys(%s)) {
      $can{$_} = $s{$_};
  }

  # Deal with regular entries.
  foreach (keys(%entry)) {
      $entry{$_} =~ s/^\s+//; # Strip unneeded spaces.
      $entry{$_} =~ s/\s+$//;

      if (defined($csa_to_canon{$_})) {
	  # Known field.
	  $can{$csa_to_canon{$_}} = $entry{$_};

      } elsif (defined($csa_special{$_})) {
	  # Special cases.
	  $can{$csa_special{$_}} = $entry{$_};
      } else {
	  # Not known.
	  $can{$_} = $entry{$_};
      }
  }
      
  # Parse names.
  $can{"Authors"} = parse_csa_names( $can{"Authors"}, $entry{"DN"} );
  $can{"Editors"} = parse_csa_names( $can{"Editors"}, $entry{"DN"} ) 
      if defined($can{"Editors"});

  %can;

}

######################################################################
# Tries to figure out type of reference. NEED more examples.
######################################################################

sub parse_csa_type {
    my ($t) = shift;
    
    if ( defined( $csa_types{$t} ) ) {
	$t = $csa_types{$t};
    } else {
	$t = "misc";
    }

    $t;
}

######################################################################
# Tries to parse CSA source. 
######################################################################

sub parse_csa_source {
    my ($s,$d,$t) = @_;
    my (%e);

    if ( defined($csa_databases_s{$d}) ) {
	%e = &{$csa_databases_s{$d}}( $s, $t );
    } elsif ( $d =~ /^Recent References/ ) { 
	# Case where it comes from some recent refs list.
	%e = parse_recent_source( $s, $t );
    } else {
	%e = parse_oceanic_source( $s, $t );
    }

    %e;
}

# These subroutines parse source for particular databases.
# For the moment, they just ignore type of reference since they all
# appear to be articles.  This will have to change in future.
sub parse_oceanic_source {
    my ($s, $t) = @_;
    my %c;
    
    # Journal name.
    if ($s =~ /^([^\[]+)\s*(\[.*\])?[,\.]\s+[Vv]ol\./) {
	$c{"Journal"} = $1;
    }

    # Volume.
    if ($s =~ /[Vv]ol\.?\s+([^,]+),/) {
	$c{"Volume"} = $1;
    }

    # Number.
    if ($s =~ /[Nn]o\.?\s+([^,]+),/) {
	$c{"Number"} = $1;
    }

    # Pages.
    if ($s =~ /p[pg]\.?\s*([^\.]+)[,\.]\s+/) {
	$c{"Pages"} = $1;
    }
    
    # Month.
    if ($s =~ /([a-zA-Z]{3})\s+[0-9]{4}\.?$/) {
	$c{"Month"} = fix_month($1);
    }
    
    %c;
}

sub parse_recent_source {
    my ($s, $t) = @_;
    my %c;
    
    # Journal name.
    if ($s =~ /^([^;]+);/) {
	$c{"Journal"} = $1;
    }

    # Volume.
    if ($s =~ /[Vv][Oo][Ll]\.?\s+([^;]+);/) {
	$c{"Volume"} = $1;
    }

    # Number.
    if ($s =~ /PART\.?\s+([^;]+);/) {
	$c{"Number"} = $1;
    }

    # Pages.
    if ($s =~ /p[pg]\.?\s*([^;]+);/) {
	$c{"Pages"} = $1;
    }
    
    # Month.
    if ($s =~ /([a-zA-Z]{3})\s+[0-9]{4}$/) {
	$c{"Month"} = fix_month($1);
    }
    
    %c;
}    

# Change from short to long form.
sub fix_month {
    my ($m) = shift;
    my ($mm) = $m;

    $m =~ tr/A-Z/a-z/;

    return $csa_months{$m} if (defined($csa_months{$m}));

    return $mm;
}

######################################################################
# Produce proper names.
######################################################################

sub parse_csa_names {
    my ($n,$d) = @_;
    if ( defined($csa_databases_n{$d}) ) {
	$n = &{$csa_databases_n{$d}}( $n );
    } else {
	$n = parse_oceanic_names( $n );
    }
 
    $n;
}

sub parse_oceanic_names {
    my ($n) = @_;
    my (@nn) = ();

    foreach (split(/;\s+/,$n)) { 
	my (@fl) = split(/,\s+/);
	
	# Deal with Junior's.
	if ($fl[1] =~ /(.+)\s+[Jj]r\.?$/) {
	    $fl[1] = $1;
	    $fl[2] = ", Jr.";
	} else {
	    $fl[2] = "";
	}
	push( @nn, bp_util::name_to_canon( "$fl[1] $fl[0]$fl[2]" ) );
    }

    $n = join( $bib::cs_sep, @nn ); # Join them together.
}

#######################
# end of package
#######################

1;
